#include "ImageBuffer.h"

#include "Texture2D.h"
#include "Core/Thread/ScopeLock.h"
#include "VHI/VHI.h"
#include "Ipc/IpcMoudle.h"

ImageBuffer* Singleton<ImageBuffer>::instance_ = nullptr;

bool ImageBuffer::Initialize() {
    return true;
}

void ImageBuffer::Uninitialize() {

}

void ImageBuffer::PushImage(ImageBuffer::IBType type, std::vector<unsigned char> data,
    uint32 width, uint32 height, uint32 comp) {
	switch (type) {
	case ImageBuffer::IBType::Human:
        PushHumanImage(std::move(data), width, height, comp);
		break;
	default:
		break;
	}
}

void ImageBuffer::Update(IBType type, Texture2D* texture) {
    switch (type) {
    case ImageBuffer::IBType::Human:
        UpdateHuman(texture);
        break;
    default:
        break;
    }
}

void ImageBuffer::PushHumanImage(std::vector<uint8> data, uint32 width, uint32 height, uint32 comp) {
    Image image{ std::move(data), width, height, comp, false };
    
    {
        ScopeLock<CriticalSection> lock(&human_cs_);
        human_.emplace(std::move(image));
       
    }
}

void ImageBuffer::UpdateHuman(Texture2D* texture) {
    int count = 0;
    {
        ScopeLock<CriticalSection> lock(&human_cs_);
        if (human_.empty()) {
            return;
        }
        count = human_.size();
    }

    if (playedIndex_ > VHI::Get()->GetAudioRender()->GetClock()+2) {
        return;
    }

    if (count > 10) {
        constexpr char full[] = "full";
        IpcMoudle::Get()->Send(full, sizeof(full) / sizeof(full[0]));
    } else if (count < 3) {
        constexpr char empty[] = "empty";
        IpcMoudle::Get()->Send(empty, sizeof(empty) / sizeof(empty[0]));
    } else {
        constexpr char normal[] = "normal";
        IpcMoudle::Get()->Send(normal, sizeof(normal) / sizeof(normal[0]));
    }

    ++playedIndex_;
    Image image;
    {
        ScopeLock<CriticalSection> lock(&human_cs_);
        image = std::move(human_.front());
        human_.pop();
    }
    
  
    texture->Update(image.width, image.height, image.comp, image.data.data());
}

void ImageBuffer::PushFaceImage(std::vector<uint8> data, uint32 width, uint32 height, uint32 comp) {
    ScopeLock<CriticalSection> lock(&face_cs_);

    face_.data = std::move(data);
    face_.width = width;
    face_.height = height;
    face_.comp = comp;
}

void ImageBuffer::UpdateFace(class Texture2D* texture) {
    if (face_.data.empty()) {
        return;
    }

    ScopeLock<CriticalSection> lock(&face_cs_);
    texture->Update(face_.width, face_.height, face_.comp, face_.data.data());
}

void ImageBuffer::PushFaceMaskImage(std::vector<uint8> data, uint32 width, uint32 height, uint32 comp) {
    ScopeLock<CriticalSection> lock(&face_mask_cs_);

    faceMask_.data = std::move(data);
    faceMask_.width = width;
    faceMask_.height = height;
    faceMask_.comp = comp;
}

void ImageBuffer::UpdateFaceMask(class Texture2D* texture) {
    if (faceMask_.data.empty()) {
        return;
    }

    ScopeLock<CriticalSection> lock(&face_mask_cs_);
    texture->Update(faceMask_.width, faceMask_.height, faceMask_.comp, faceMask_.data.data());
}
